@@ -14,8 +14,8 @@
  *
  * You should have received a copy of the GNU Library General Public
  * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA 02110-1301, USA.
  */
 
 #include <stdio.h>
@@ -125,7 +125,7 @@ run_test (void *threadid)
       mess_some_more ();
       if (g_timer_elapsed (timer, NULL) > 0.5) {
         g_mutex_lock (&fdlock);
-        g_print ("active fds :%d\n", g_list_length (fds));
+        g_print ("active fds :%u\n", g_list_length (fds));
         g_timer_start (timer);
         g_mutex_unlock (&fdlock);
       }
@@ -165,6 +165,7 @@ main (gint argc, gchar * argv[])
 
     if (error) {
       printf ("ERROR: g_thread_try_new() %s\n", error->message);
+      g_clear_error (&error);
       exit (-1);
     }
   }
