@@ -39,7 +39,7 @@ printf_extension_log_func (GstDebugCategory * category,
   dbg_msg = gst_debug_message_get (message);
   fail_unless (dbg_msg != NULL);
 
-  if (save_messages)
+  if (save_messages && g_str_equal (category->name, "check"))
     messages = g_list_append (messages, g_strdup (dbg_msg));
 
   /* g_print ("%s\n", dbg_msg); */
@@ -89,6 +89,24 @@ GST_START_TEST (info_ptr_format_printf_extension)
     gst_message_unref (msg);
   }
 
+  /* buffer and buffer list */
+  {
+    GstBufferList *list;
+    GstBuffer *buf;
+
+    buf = gst_buffer_new_allocate (NULL, 42, NULL);
+    GST_BUFFER_PTS (buf) = 5 * GST_SECOND;
+    GST_BUFFER_DURATION (buf) = GST_SECOND;
+    GST_LOG ("BUFFER: %" GST_PTR_FORMAT, buf);
+
+    list = gst_buffer_list_new ();
+    gst_buffer_list_add (list, buf);
+    buf = gst_buffer_new_allocate (NULL, 58, NULL);
+    gst_buffer_list_add (list, buf);
+    GST_LOG ("BUFFERLIST: %" GST_PTR_FORMAT, list);
+    gst_buffer_list_unref (list);
+  }
+
 #if 0
   /* TODO: GObject */
   {
@@ -222,7 +240,7 @@ GST_START_TEST (info_dump_mem)
     0x00, 0x00, 0x00, 0x00, 0x00, 0x01, '%', 's', '%', 's'
   };
 
-  e = gst_element_factory_make ("fakesink", NULL);
+  e = gst_pipeline_new ("pipeline");
   GST_DEBUG_CATEGORY_INIT (cat, "dumpcat", 0, "data dump debug category");
   GST_MEMDUMP ("quicktime header", data, sizeof (data));
   GST_MEMDUMP (NULL, data, sizeof (data));
@@ -239,7 +257,7 @@ GST_START_TEST (info_fixme)
   GstDebugCategory *cat = NULL;
   GstElement *e;
 
-  e = gst_element_factory_make ("fakesink", NULL);
+  e = gst_pipeline_new ("pipeline");
   GST_DEBUG_CATEGORY_INIT (cat, "fixcat", 0, "FIXME debug category");
   GST_FIXME ("fix %s thing", "this");
   GST_FIXME_OBJECT (e, "fix %s object", "this");
@@ -309,7 +327,7 @@ GST_START_TEST (info_old_printf_extensions)
   gst_debug_add_log_function (gst_debug_log_default, NULL, NULL);
   gst_debug_remove_log_function (printf_extension_log_func);
   save_messages = FALSE;
-  g_list_foreach (messages, (GFunc) g_free, NULL);
+  g_list_free_full (messages, (GDestroyNotify) g_free);
   messages = NULL;
 }
 
@@ -329,9 +347,81 @@ GST_START_TEST (info_register_same_debug_category_twice)
       "Going once");
 }
 
+GST_END_TEST;
+
+GST_START_TEST (info_set_and_unset_single)
+{
+  GstDebugLevel orig = gst_debug_get_default_threshold ();
+  GstDebugLevel cat1, cat2;
+  GstDebugCategory *states;
+
+  GST_DEBUG_CATEGORY_GET (states, "GST_STATES");
+  fail_unless (states != NULL);
+
+  gst_debug_set_default_threshold (GST_LEVEL_WARNING);
+
+  gst_debug_set_threshold_for_name ("GST_STATES", GST_LEVEL_DEBUG);
+  cat1 = gst_debug_category_get_threshold (states);
+  gst_debug_unset_threshold_for_name ("GST_STATES");
+  cat2 = gst_debug_category_get_threshold (states);
+
+  gst_debug_set_default_threshold (orig);
+  fail_unless (cat1 = GST_LEVEL_DEBUG);
+  fail_unless (cat2 = GST_LEVEL_WARNING);
+}
+
+GST_END_TEST;
+
+GST_START_TEST (info_set_and_unset_multiple)
+{
+  GstDebugLevel orig = gst_debug_get_default_threshold ();
+  GstDebugLevel cat1, cat2, cat3;
+  GstDebugCategory *states;
+  GstDebugCategory *caps;
+
+  GST_DEBUG_CATEGORY_GET (states, "GST_STATES");
+  GST_DEBUG_CATEGORY_GET (caps, "GST_CAPS");
+  fail_unless (states != NULL);
+  fail_unless (caps != NULL);
+
+  gst_debug_set_default_threshold (GST_LEVEL_WARNING);
+
+  gst_debug_set_threshold_for_name ("GST_STATES", GST_LEVEL_DEBUG);
+  gst_debug_set_threshold_for_name ("GST_CAPS", GST_LEVEL_DEBUG);
+  cat1 = gst_debug_category_get_threshold (states);
+  gst_debug_unset_threshold_for_name ("GST_STATES");
+  gst_debug_unset_threshold_for_name ("GST_CAPS");
+  cat2 = gst_debug_category_get_threshold (states);
+  cat3 = gst_debug_category_get_threshold (caps);
+
+  gst_debug_set_default_threshold (orig);
+
+  fail_unless (cat1 = GST_LEVEL_DEBUG);
+  fail_unless (cat2 = GST_LEVEL_WARNING);
+  fail_unless (cat3 = GST_LEVEL_WARNING);
+}
+
 GST_END_TEST;
 #endif
 
+GST_START_TEST (info_fourcc)
+{
+  gchar *res;
+  const gchar *cmp;
+
+  cmp = "abcd";
+  res = g_strdup_printf ("%" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (0x64636261));
+  fail_unless_equals_string (res, cmp);
+  g_free (res);
+
+  cmp = ".bcd";
+  res = g_strdup_printf ("%" GST_FOURCC_FORMAT, GST_FOURCC_ARGS (0x646362a9));
+  fail_unless_equals_string (res, cmp);
+  g_free (res);
+}
+
+GST_END_TEST;
+
 static Suite *
 gst_info_suite (void)
 {
@@ -341,6 +431,7 @@ gst_info_suite (void)
   tcase_set_timeout (tc_chain, 30);
 
   suite_add_tcase (s, tc_chain);
+  tcase_add_test (tc_chain, info_fourcc);
 #ifndef GST_DISABLE_GST_DEBUG
   tcase_add_test (tc_chain, info_segment_format_printf_extension);
   tcase_add_test (tc_chain, info_ptr_format_printf_extension);
@@ -349,6 +440,8 @@ gst_info_suite (void)
   tcase_add_test (tc_chain, info_fixme);
   tcase_add_test (tc_chain, info_old_printf_extensions);
   tcase_add_test (tc_chain, info_register_same_debug_category_twice);
+  tcase_add_test (tc_chain, info_set_and_unset_single);
+  tcase_add_test (tc_chain, info_set_and_unset_multiple);
 #endif
 
   return s;
