@@ -81,8 +81,8 @@ gst_protection_test_base_init (GstProtectionTestClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_pad_template (element_class,
-      gst_static_pad_template_get (&gst_decrypt_sink_template));
+  gst_element_class_add_static_pad_template (element_class,
+      &gst_decrypt_sink_template);
 
   gst_element_class_set_metadata (element_class,
       "Decryptor element for unit tests",
